use anyhow::Result;

pub(super) mod syntax {
	use serde::Deserialize;
	// Raw serde syntax structs.
	// These are never seen by code outside this crate.

	#[derive(Debug, Deserialize)]
	pub struct Engine {
		pub thrust: f32,
		pub flare: Flare,
	}

	#[derive(Debug, Deserialize)]
	pub struct Flare {
		pub sprite: String,
	}
}

/// Represents an (foward) engine outfit that may be attached to a ship.
#[derive(Debug, Clone)]
pub struct Engine {
	/// The name of this outfit
	pub name: String,

	/// How much thrust this engine produces
	pub thrust: f32,

	/// The flare sprite this engine creates.
	/// Its location and size is determined by a ship's
	/// engine points.
	pub flare_sprite: String,
}

impl super::Build for Engine {
	fn build(root: &super::syntax::Root) -> Result<Vec<Self>> {
		let engine = if let Some(engine) = &root.engine {
			engine
		} else {
			return Ok(vec![]);
		};

		let mut out = Vec::new();
		for (engine_name, engine) in engine {
			out.push(Self {
				name: engine_name.to_owned(),
				thrust: engine.thrust,
				flare_sprite: engine.flare.sprite.clone(),
			});
		}

		return Ok(out);
	}
}
