use cgmath::{Deg, InnerSpace};

use crate::{
	inputstatus::InputStatus,
	physics::{util, Physics, ShipHandle},
};

pub trait ShipBehavior
where
	Self: Send,
{
	fn update_controls(&mut self, physics: &mut Physics, input: &InputStatus, player: ShipHandle);
}

pub struct Dummy {
	_handle: ShipHandle,
}

impl Dummy {
	pub fn new(handle: ShipHandle) -> Box<Self> {
		Box::new(Self { _handle: handle })
	}
}

impl ShipBehavior for Dummy {
	fn update_controls(
		&mut self,
		_physics: &mut Physics,
		_input: &InputStatus,
		_player: ShipHandle,
	) {
	}
}

pub struct Player {
	handle: ShipHandle,
}

impl Player {
	pub fn new(handle: ShipHandle) -> Box<Self> {
		Box::new(Self { handle })
	}
}

impl ShipBehavior for Player {
	fn update_controls(&mut self, physics: &mut Physics, input: &InputStatus, _player: ShipHandle) {
		let s = physics.get_ship_mut(&self.handle);
		s.controls.left = input.key_left;
		s.controls.right = input.key_right;
		s.controls.guns = input.key_guns;
		s.controls.thrust = input.key_thrust;
	}
}

pub struct Point {
	handle: ShipHandle,
}

impl Point {
	pub fn new(handle: ShipHandle) -> Box<Self> {
		Box::new(Self { handle })
	}
}

impl ShipBehavior for Point {
	fn update_controls(&mut self, physics: &mut Physics, _input: &InputStatus, player: ShipHandle) {
		let (_, r) = physics.get_ship_body(&player);
		let p = util::rigidbody_position(r);

		let (_, r) = physics.get_ship_body(&self.handle);
		let t = util::rigidbody_position(r);
		let pa = util::rigidbody_rotation(r);
		let v = r.angvel();

		let d: Deg<f32> = (t - p).angle(pa).into();
		println!("{:?}", d);

		let s = physics.get_ship_mut(&self.handle);

		s.controls.left = false;
		s.controls.right = false;

		if d < Deg(0.0) && v < 0.1 {
			s.controls.left = false;
			s.controls.right = true;
			println!("r")
		} else if d > Deg(0.0) && v > -0.1 {
			println!("l");
			s.controls.left = true;
			s.controls.right = false;
		}

		s.controls.guns = true;
		s.controls.thrust = false;
	}
}
